package CheckRelease2::CMS::Comment;

use strict;

use MT;
use CheckRelease2::Approval;
use CheckRelease2::Util qw( _t plugin can_create add_alt_tmpl_path_mtcloud);
use File::Spec;
use Data::Dumper;

sub list {
	my $app = shift;
	my $q = $app->param;
	my $user = $app->user;
	my $type = $q->param('_type');
	my $blog = $app->blog;
	my $blog_id = $blog->id;
	my $id = $q->param('id');

	return $app->error( $app->translate('Permission denied') )
		unless can_create($user, $type, $blog);

	return $app->error( $app->translate('Invalid request') )
		unless $id =~ m!^\d+$!;

	my $terms = { object_id => $id };
	if ( $type eq 'entry' || $type eq 'page' ) {
		$terms->{object_class} = $type ;		
	} else {
		$terms->{content_type_id} = $type;
	}
	my $approval = $app->model('cr2_approval')->load($terms);
	my $args = { sort => 'created_on', direction => 'descend' };
	my @comments = defined $approval
		? $app->model('cr2_comment')->load({ cr2_approval_id => $approval->id}, $args)
		: ();

	my @author_ids = map { $_->created_by } @comments;
	my %authors = map { $_->id => $_ } $app->model('author')->load({ id => \@author_ids });

	my @objects;
	foreach my $comment ( @comments ) {
		my $created_by = $comment->created_by;
		my $user = defined $authors{ $comment->created_by } ? $authors{ $comment->created_by } : undef;
		my $row = {
			id => $comment->id,
			body => $comment->comment,
			created_on => $comment->created_on,
			status => $comment->status,
		};
		if ( defined $user ) {
			$row->{nickname} = $user->nickname;
			$row->{author_id} = $user->id;
			$row->{userpic} = $user->userpic_url;
		}
		push @objects, $row;
	}
	my $params = { comments => \@objects };

	my $plugin = plugin();
  $app->{plugin_template_path} = File::Spec->catfile($plugin->envelope, 'tmpl', 'cms');
  add_alt_tmpl_path_mtcloud();
	my $tmpl = 'dialog/list_comment.tmpl';
	return $app->load_tmpl($tmpl, $params);
}

1;