package CheckRelease2::CMS::Step;

use strict;
use MT;
use CheckRelease2::Util qw(plugin unable_plugin _t do_redirect lookup_rule sanitize can_manage_workflow add_alt_tmpl_path_mtcloud);
use JSON;
use File::Spec;

sub has_permission {
	my $app = shift;
	my $q = $app->param;
	my $blog_id = $q->param('blog_id');
	return $app->return_to_dashboard() if !$blog_id || unable_plugin();
	return $app->permission_denied()
		unless can_manage_workflow($app->user, $blog_id);	
	1;
}

sub _hdlr_cms_view_permission_filter {
	my ($cb, $app, $id) = @_;
	has_permission($app) or return;
	1;
}

sub save {
	my $app = shift;
	my $q = $app->param;
	my $blog_id = $q->param('blog_id');
	my $plugin = plugin();

	has_permission($app) or return;
	$app->validate_magic || !$q->param('name')
		or return $app->error( _t('Invalid request') );

	my $id = $q->param('id');
	return $app->error( _t('Invalid request') ) if $id && $id !~ m!^\d+$!;
	my $model = $app->model('cr2_step');
	my $step = $id ? $model->load({ id => $id }) : $model->new();
	$step->blog_id( $blog_id );
	$step->name( sanitize($q->param('name')) );
	$step->steps( sanitize($q->param('steps')) );
	unless ( $step->save() ) {
		my $params = {
			blog_id => $blog_id,
			name => $q->param('name'),
			steps => $q->param('steps'),
		};
  	$app->{plugin_template_path} = File::Spec->catfile($plugin->envelope, 'tmpl', 'cms');
  	add_alt_tmpl_path_mtcloud();
		my $tmpl = 'edit_cr2_step.tmpl';
		return $app->load_tmpl($tmpl, $params);
	}

	$app->redirect(
		$app->uri(mode => 'view',
			args => {
				_type => 'cr2_step',
				blog_id => $blog_id,
				id => $step->id,
				saved => 1,
			}
		)
	);
}

sub edit_rules {
	my $app = shift;
	my $q = $app->param;
	my $blog_id = $q->param('blog_id');
	my $plugin = plugin();
	has_permission($app) or return;

	my @steps = $app->model('cr2_step')->load();
	my $params = { objects => [], workflows => [] };
	$params->{saved} = $q->param('saved') ? 1 : 0;

	my $terms = { blog_id => $blog_id };
	my %rules = map {
		my $key = defined $_->object_id ? $_->object_id : $_->object_ds;
		$key => $_;
	} $app->model('cr2_rule')->load($terms);

	foreach ( qw/entry page/ ) {
		push @{ $params->{objects} }, {
			id => $_,
			unique_id => $_,
			name => $app->translate($_),
			wf_id => defined( $rules{$_} ) ? $rules{$_}->step_id : '',
		};
	}
	my $iter = $app->model('content_type')->load_iter({ blog_id => $blog_id });
	my @ct = $app->model('content_type')->load({ blog_id => $blog_id });
	while ( $_ = $iter->() ) {
		push @{ $params->{objects} }, {
			id => $_->id,
			unique_id => $_->unique_id,
			name => $_->name,
			wf_id => defined( $rules{$_->id} ) ? $rules{$_->id}->step_id : '',
		};
	}
	foreach ( @steps ) {
		push @{ $params->{workflows} }, {
			id => $_->id,
			name => $_->name
		};
	}

  $app->{plugin_template_path} = File::Spec->catfile($plugin->envelope, 'tmpl', 'cms');
  add_alt_tmpl_path_mtcloud();
	my $tmpl = 'edit_rules.tmpl';
	return $app->load_tmpl($tmpl, $params);
}

sub clear_rules {
	my $app = MT->instance;
	my $q = $app->param;
	my $blog_id = $q->param('blog_id');
	my $class = $app->model('cr2_rule');
	my $terms = {
		blog_id => $app->param('blog_id'),
	};
	my @rules = $class->load($terms);
	my $res = 1;
	foreach my $rule ( @rules ) {
		unless ( $rule->remove() ) {
			MT::log({
				blog_id => $blog_id,
				level => MT::Log::ERROR,
				message => _t('Failure to remove an approval rule. [_1]', $rule->errstr)
			});
			$res = 0;
		}
	}
	$res;
}

sub save_rules {
	my $app = shift;
	my $q = $app->param;
	my $blog_id = $q->param('blog_id');

	clear_rules();

	my @rules;
	my %params = $app->param_hash;
	my $class = $app->model('cr2_rule');
	foreach my $key ( keys %params ) {
		next unless $key =~ m!^workflow_(.+)$!;
		my $rule = lookup_rule($blog_id, $1);
		my $old_type = ( $1 eq 'entry' || $1 eq 'page' ) ? $1 : undef;
		my $object_id = $1;
		my $val = $params{$key} || '';
		if ( defined $rule ) {
			if ( !$val ) {
				$rule->remove();
				next;	
			} elsif ( $rule->step_id == $val ) {
				next;
			}
		}
		next unless $val;
		
		$rule = $class->new();
		$rule->blog_id( $blog_id );
		if ( defined $old_type ) {
			$rule->object_ds( $old_type );
		} else {
			$rule->object_id( $object_id || 0 );
		}
		$rule->step_id( $val );
		$rule->save()
			or die $app->error( $rule->errstr );
	}

	my $uri = $app->uri(
		mode => 'rules',
		args => {
			blog_id => $blog_id,
			_type => 'cr2_step',
			saved => 1,
		}
	);
	do_redirect($app, $uri);
}

sub _hdlr_template_source_header {
  my ( $cb, $app, $tmpl ) = @_;
  my $q = $app->param;
  my $mode = $q->param('__mode') || '';
  my $type = $q->param('_type') || '';
  return 1
    if !($mode eq 'list' && $type eq 'cr2_step');
   my $src = '<link rel="stylesheet" href="' .$app->static_path .'plugins/CheckRelease2/css/list.css">';
   $$tmpl =~ s!(<mt:var name="html_head">)!$src$1!m;
   $$tmpl;
}
1;