package CheckRelease2::ListProperty;

use strict;

use CheckRelease2::Util qw(plugin _t unable_plugin lookup_approval lookup_rule lookup_step load_latest_revision get_config_value unserialize);
use CheckRelease2::ApprovalStatus qw(:all);
use MT::EntryStatus qw(:all);
use MT::Entry;
use Data::Dumper;

sub _hdlr_condition {
	my $app = MT->instance;
	my $user = $app->user;
	return 0 if unable_plugin();
	my %params = $app->param_hash;
  my $q = $app->param;
  my $type = $q->param('_type') || '';
  my $blog_id = $q->param('blog_id');
  my $sub_type = $q->param('type') ? $q->param('type') : undef;
  if ( !($type eq 'entry' || $type eq 'page') && !defined $sub_type ) {
    my $ds = $params{datasource};
    $type ||= $ds || '';
  }
  if ( $sub_type =~ m!content_data_(\d+)$! || $type =~ m!content_data_(\d+)$! ) {
    $type = $1;
  }
	return 0 unless lookup_rule($blog_id, $type);
	1;
}

sub _hdlr_entry_approval_status_html {
	my $prop = shift;
	my ($obj, $app) = @_;
	return undef if unable_plugin();
	my $terms = {
		object_class => $obj->class,
		object_id => $obj->id,
	};
	my $approval = $app->model('cr2_approval')->load($terms);
	my $status = !defined($approval) ? NOT_COMPATIBLE : $approval->status;
	my $html = '';
	if ( $status == NOT_COMPATIBLE ) {
		$html = 'not compatible';
	} elsif ( $status == CHECKING ) {
		$html = 'CHECKING';
	} elsif ( $status == REMAND ) {
		$html = 'REMAND';
	} elsif ( $status == APPROVED ) {
		$html = 'APPROVED';
	} elsif ( $status == DISMISSAL ) {
		$html = 'DISMISSAL';
	}
	$html;
}

sub _hdlr_entry_revision_html {
	my ($prop, $obj, $app, $condition) = @_;
	return '' if unable_plugin();
	return '' unless ref $app->blog;
	my $user = $app->user;
	my $class = $obj->class;
	$class = $obj->content_type_id if $class =~ m!^content_data!;
	my $rev = load_latest_revision($app, $class, $obj->id);
	my $approval = lookup_approval($obj->id, $class);
	my $can_create = 0;
	my $has_new_rev = 0;
	my ($is_rev, $rev_number, $rev_status, $current_step);
	if ( defined $approval ) {
		$approval->is_rev(1);
		$is_rev = 1;
		$rev_number = $approval->rev_number || 0;
		$rev_status = $approval->rev_status;
		$current_step = $approval->rev_current_step;
	}

	if ( $obj->status == MT::Entry::RELEASE() ) {
		$can_create = 1;
	}
	if ( $obj->current_revision < $rev_number ) {
		$has_new_rev = 1;
	}
	my $html = $has_new_rev ? edit_latest_rev($app, $obj) : '';
	$html .=	$can_create ? create_new_rev($app, $obj) : '' unless $html;
	if ( $html ) {
		my $blog_id = $app->param('blog_id');
	  my $rule = lookup_rule($blog_id, $class)
	    or return $html;
	  my $step = lookup_step($rule->step_id);
  	my $current = $is_rev && !$rev_number ? '-'	: $current_step;
  	if ( !$current && $has_new_rev ) {
  		$current = !$current ? 1 : '-';
  	} else {
  		$current++;
  	}
	  $html = approval_status_label( $rev_status, $current, @{$step->step_tree} + 1)
	  	.'<br>'.$html;
	}
	$html;
}

sub create_new_rev {
	my ($app, $obj) = @_;
	my $q = $app->param;
	my $class = $obj->class;
	my $type = $class;
	$class = $obj->content_type_id if $class =~ m!^content_data!;
	my $rev = load_latest_revision( $app, $class, $obj->id );
	my $rev_num = defined($rev) ? $rev->rev_number : -1;
	my @args = (
		'__mode=view',
		'blog_id='.$q->param('blog_id'),
		'_type='.$type,
		'id='.$obj->id,
		'r=' .$rev_num,
		'crr=-1'
	);
	if ( !($type eq 'entry' || $type eq 'page') ) {
		push @args, 'content_type_id='.$class;
	}
	my $uri = $app->mt_uri .'?' .join('&', @args);
	build_icon_link($uri, 'cr2-list-new', '', $app->config->StaticWebPath, 'plus', _t('New'));
}

sub edit_latest_rev {
	my ($app, $obj) = @_;
	my $q = $app->param;
	my $class = $obj->class;
	my $type = $class;
	$class = $obj->content_type_id if $class =~ m!^content_data!;
	my $rev = load_latest_revision( $app, $class, $obj->id );
	my $approval = lookup_approval($obj->id, $class, $rev->rev_number);
	return if !defined($approval) || (defined($approval) && !$approval->rev_number);
	my $rev_num = defined($rev) ? $rev->rev_number : -1;
	my @args = (
		'__mode=view',
		'blog_id='.$q->param('blog_id'),
		'_type='.$type,
		'id='.$obj->id,
		'r=' .$rev_num,
		'crr=1'
	);
	if ( !($type eq 'entry' || $type eq 'page') ) {
		push @args, 'content_type_id='.$class;
	}
	my $uri = $app->mt_uri .'?' .join('&', @args);
	my $status = 'draft';
	my $data_col = $type eq 'content_data' ? 'cd' : 'entry'; 
	if ( defined $rev ) {
		my $unpacked = unserialize($rev->$data_col);
		$status = 'clock' if $$unpacked->{status} == FUTURE;
	}
	build_icon_link($uri, 'cr2-list-edit', $status eq 'clock' ? 'mt-icon--info' : '', $app->config->StaticWebPath, $status, _t('Edit'), $rev_num);
}

sub build_icon_link {
	my ($uri, $cls, $icon_cls, $static_path, $status, $label, $rev_num) = @_;
	my $html = '<a href="'.$uri .'" class="'. $cls .'"><svg role="img" class="mt-icon mt-icon--sm ' .$icon_cls .'"><use xlink:href="' 
		.$static_path .'images/sprite.svg#ic_'.$status .'"></use></svg>' .$label;
	if ( defined $rev_num ) {
		$html .= '(' .$rev_num .')';
	}
	$html .= '</a>';
	$html;
}

sub _hdlr_entry_approval_status_html {
	my ($prop, $obj, $app, $condition) = @_;
	return '' if unable_plugin();
	return '' unless ref $app->blog;
	my $user = $app->user;
	my $class = $obj->class;
	$class = $obj->content_type_id if $class =~ m!^content_data!;
  my $rule = lookup_rule($app->param('blog_id'), $class)
    or return '';
  my $step = lookup_step($rule->step_id);
	my $approval = lookup_approval($obj->id, $class);
	return approval_status_label(0, '-', @{$step->step_tree} + 1) unless defined $approval;
  my $current = $approval->current_step + 1;
	approval_status_label($approval->status, $current, @{$step->step_tree} + 1);
}

sub approval_status_label {
	my ($status, $current, $max) = @_;
	my $class = $status == CHECKING
		? 'checking'
		: $status == REMAND
			? 'remand'
			: $status == APPROVED
				? 'approved'
				: $status == DISMISSAL
					? 'dismissal'
					: 'not-compatible';
	my $html = '<div class="cr2-approval-status cr2-' .$class .'">';
	$html .= _t($class);
  $html .= ' (' .$current .'/' .$max .')';
	$html .= '</div>';
	$html;
}
1;