package CheckRelease2::Step;

use strict;
use base qw( MT::Object );
use CheckRelease2::Util qw(_t);
use MT::Util;

__PACKAGE__->install_properties({
  column_defs => {
    id      => 'integer not null auto_increment',
    blog_id => 'integer',
    name    => 'string(255)',
    steps   => 'text',
  },
  index => {
  	blog_id => 1,
  },
  datasource => 'cr2_step',
  primary_key => 'id',
  audit => 1,
});

sub list_prop {
  my $props = {
    name => {
      auto => 1,
      order => 100,
      label => 'Name',
      display => 'force',
      html_link => sub {
        my ( $prop, $obj, $app ) = @_;
        my $user = $app->user;
        my $blog_id = $obj->blog_id || 0;

        return $app->return_to_dashboard( permission => 1 )
          unless $app->user->permissions($blog_id)->permissions =~ m!(edit_cr2_step|administer)!; 

        return $app->uri(
            mode => 'view',
            args => {
                _type => 'cr2_step',
                id => $obj->id,
                $obj->blog_id ? ( blog_id => $obj->blog_id ) : (),
            },
        );
      },
    },
    summary => {
        label => 'Summary',
        order => 400,
        display => 'force',
        html => sub {
            my ( $prop, $obj, $app ) = @_;
            return '' unless $obj->steps;
            require MT::Util;
            my $steps = MT::Util::from_json($obj->steps);
            my $step_num = scalar @$steps;
            my $auth_num = 0;
            foreach ( @$steps ) {
              next unless @{ $_ };
              $auth_num += @{ $_ };
            }
            &_build_html_step_summary($step_num, $auth_num, $steps);
        },
    },
  };
  $props;
}

sub _build_html_step_summary {
  my ($step_num, $auth_num, $steps) = @_;
  #TODO return list step summery num
  my $html .= '<div class="cr2_steps">';
  $html .= ($step_num + 1) ._t(' step');
  $html .= '</div>';
  $html .= '<div class="cr2_step_list"><div class="cr2_step_item">' ._t('Author') .'</div></div>';

  my %ids = map { map { $_ => 1 } @$_ } @$steps;
  my %authors;
  my $iter = MT->model('author')->load_iter({ id => [keys %ids] });
  while ( my $author = $iter->() ) {
    $authors{ $author->id } = $author;
  }
  my @step_html;
  foreach my $step ( @$steps ) {
    my @approvers;
    foreach ( @$step ) {
      next unless defined $authors{ $_ };
      my $author = $authors{ $_ };
      push @approvers, $author->nickname .'(ID: ' .$author->id .')';
    }
    push @step_html, '<div class="cr2_step_list"><div class="cr2_step_item">' .join(', ', @approvers) .'</div></div>';
  }
  $html .= join('', @step_html);
  $html;
}

sub step_tree {
  my $self = shift;
  MT::Util::from_json($self->steps);
}

sub can_view_workflow_list {
  my $app = shift;
  1;
}
1;