package CheckRelease2::Task;

# プラグマ
use strict;

# モジュール
use File::Spec;
use MT;
use MT::Util;
use Data::Dumper;
use CustomFields::Util qw(save_meta);
use CheckRelease2::ApprovalStatus qw(:all);
use CheckRelease2::Util qw(unable_plugin lookup_approval unserialize now);
use MT::EntryStatus qw(:all);

sub future_post {
  my $task = shift;
  return 1 if unable_plugin();
  
  my $prefix = 'cr2_future_que';
  my $class = MT->instance->model('session');
  my %blogs;
  my %authors;
  my $place_class = MT->model('placement');
  my $author_class = MT->model('author');
  my $tag_class = MT->model('tag');
  my $oa_class = MT->model('objectasset');
  foreach ( qw/entry page cd/ ) {
    my $type = $_;
    my $id_col = $type .'_id';
    my $name = join(':', ($prefix, $type, '%'));
    my @sess = $class->load({ name => { 'like' => $name }, kind => 'CR' });
    next unless @sess;
    my %sess;
    my %objs;
    my @ids = map {
      my $id = $_->get('object_id');
      my $rev_id = $_->get('revision_id');
      unless ( defined $rev_id ) {
      	(undef, undef, $id, $rev_id) = split /:/, $_->name;

      }
      # the reision is not latest, pass
    	if ( defined $objs{$id} ) {
    		my (undef, undef, $pid, $prev_id) = split /:/, $objs{$id}->name;
    		if ( $prev_id > $rev_id ) {
      		$sess{ $_->name } = $_;
      		$rev_id = $prev_id;
      	}
    	}
 			$objs{$id} = $_;
      $sess{ $_->name } = $_;
      $rev_id;
    } @sess;
    next unless @ids;

    my $iter = MT->model($type .':revision')->load_iter({ id => \@ids }, { sort => 'id', direction => 'ascend'})
      or next;
    my $now = now;
    my $cols = [];
    while ( my $rev = $iter->() ) {
      my $unpacked = unserialize( $rev->$type );
      if ( !$$unpacked->{authored_on} || $now < $$unpacked->{authored_on} || $$unpacked->{status} != MT::Entry::FUTURE() ) {
        delete $sess{ join(':', ($prefix, $type, $$unpacked->{id}, $rev->id)) };
        next;
      }

      my $obj = MT->model($type)->load({ id => $$unpacked->{id} });
      unless ( $obj ) {
        my $sess = $sess{ join(':', ($prefix, $type, $$unpacked->{id}, $rev->id)) };
        defined($sess) && $sess->remove();
        next;
      }
      $cols = $obj->column_names unless scalar @$cols;
      
      my $blog = $obj->blog;
      $blogs{ $blog->id } = $blog unless defined $blogs{ $blog->id };
      
      map { $obj->$_( $$unpacked->{$_} ) } @$cols;
      $obj->status( RELEASE() );

      my ($meta, $orig_obj_asset, $orig_pls);
      my $new_pls = [];
      my $new_obj_asset = [];
      if ( $type eq 'entry' || $type eq 'page' ) {
        $meta = $obj->meta;
        foreach ( keys %$$unpacked ) {
          if ( /^field\./ ) {
            $meta->{$_} = defined($$unpacked->{$_}) ? $$unpacked->{$_} : undef;
          }
        }
        
        my $author_id = $rev->modified_by || $rev->created_by;
        my $user = defined($authors{ $author_id })
          ? $authors{ $author_id } : undef;
        my $delim = ',';
        unless ( defined $user ) {
          $user = $author_class->load({ id => $author_id });
          $authors{ $author_id } = $user if ref $user;
          $delim = $user->entry_prefs->{tag_delim} if ref $user;
        }

        my $orig_pls;
        my @cat_ids = map { $_->[0] } @{ $$unpacked->{__rev_cats} };
        my $orig_pls_hash = [];
        my $pl_cols = [];
        my @pls = $place_class->load( { entry_id => $obj->id } );
        $orig_pls = \@pls if \@pls;
        if ( @cat_ids ) {
          my $cntr = 0;
          foreach my $cat_id ( @cat_ids ) {
            my $p = $place_class->new;
            $p->blog_id( $$unpacked->{blog_id} );
            $p->category_id( $cat_id );
            $p->$id_col( $$unpacked->{id} );
            $p->is_primary( $cntr++ == 0 ? 1 : 0 );
            push @$new_pls, $p;
          }
        }
        
        my $object_asset = $$unpacked->{objectasset};
        my $orig_obj_asset;
        my $orig_obj_asset_hash = [];
        my $oa_cols = [];
        if ( @$object_asset ) {
          my $oa_terms;
          $oa_terms->{object_ds} = $type;
          $oa_terms->{object_id} = $obj->id;
          my @oa = $oa_class->load( $oa_terms );
          $orig_obj_asset = \@oa if @oa;
          
          foreach my $oa_pack ( @$object_asset ) {
            my $oa = $oa_class->new;
            $oa_cols = $oa->column_names unless @$oa_cols;
            $oa->$_( $oa_pack->{$_} ) foreach @$oa_cols;
            push @$new_obj_asset, $oa;
          }
        }
        
        my $tags = $$unpacked->{__rev_tags} || undef;
        my @tags;
        if ( defined $tags ) {
          @tags = map { $_->name } $tag_class->load({ id => $tags });
        }
        
        if ( @tags ) {
          $obj->set_tags( @tags );
        } else {
          $obj->remove_tags();
        }
        $obj->meta( $meta );
      }
      my $latest_rev = load_latest_revision($obj->datasource, $obj->id);
      $obj->current_revision( $latest_rev->rev_number ) if ref $latest_rev;
      $obj->save()
        or next;
      if ( $type eq 'entry' || $type eq 'page' ) {
        save_meta( $obj, $meta );
        $_->remove() foreach @$orig_pls;
        $_->save() foreach @$new_pls;
        my @oa_clones = map { $_->clone } @$orig_obj_asset;
        $_->remove() foreach @$orig_obj_asset;
        $_->save() foreach @$new_obj_asset;
      }
      eval {
        if ( $type eq 'entry' || $type eq 'page' ) {
          MT->rebuild_entry( Entry => $obj, Blog => $blog )
            or die MT->errstr;
        } else {
          MT->rebuild_content_data(
              ContentData       => $obj,
              BuildDependencies => 1,
          );
        }
      };
      if ( $@ ) {
        require MT::Log;
        MT->log({
          message => MT->translate('An error occurred while publishing scheduled entries: [_1]', $@),
          class   => "system",
          blog_id => $blog->id,
          level   => MT::Log::ERROR()
        });
      }
      my $sess = $sess{ join(':', ($prefix, $type, $$unpacked->{id}, $rev->id)) };
      defined($sess) && $sess->remove();

      my $approval = lookup_approval($obj->id, $obj->class eq 'content_data' ? $obj->content_type_id : $obj->class);
      next unless defined $approval;
      $approval->is_rev(1);
      $approval->status( $approval->get_status );
      $approval->current_step( $approval->get_current_step );
      $approval->rev_status( NOT_COMPATIBLE );
      $approval->rev_current_step( 0 );
      $approval->rev_number( undef );
      $approval->save();
    }

    # clean up old task is not processed
    foreach ( keys %sess ) {
    	$sess{$_}->remove();
    }
  }
  foreach ( %blogs ) {
    MT->rebuild_indexes( Blog => $blogs{$_} );
  }
  1;
}

sub to_hash {
  my ($objs) = @_;
  my $cols = [];
  return [] unless @$objs;
  my $hashes = [];
  foreach my $obj ( @$objs ) {
    $cols = $obj->column_names unless @$cols;
    my %row = map { $_ => $obj->$_ } grep { $_ ne 'id' } @$cols;
    push @$hashes, \%row;
  }
  $hashes;
}

sub load_latest_revision {
  my ($class, $id) = @_;
  my $pkg = 'MT::Entry::Revision';
  return undef unless $pkg;
  my $terms;
  $terms->{$class .'_id'} = $id;
  my $args;
  $args->{limit} = 1;
  $args->{'sort'} = 'rev_number';
  $args->{direction} = 'descend';
  my @revs = $pkg->load($terms, $args);
  @revs ? shift @revs : undef;
}


1;